#version 150

varying vec2 vTextureCoord;
varying vec4 vPosition;

uniform sampler2D uDiffuseSampler;
uniform sampler2D uDepthSampler;
uniform float uAlphaMultiplier;

uniform vec2 uBlurDirection;
uniform float uBlurAmount;
uniform float uDepthBias;
uniform float uNearPlane;
uniform float uFarPlane;

uniform float uFocusEnd;
uniform float uFocusStart;
uniform float uFalloff;

uniform vec2 uScreenSize;

void main(void)
{
		vec4 baseSample = texture2D(uDiffuseSampler, vTextureCoord);
				
		vec4 depth = texture2D(uDepthSampler, vTextureCoord);
		float z_n = 2.0 * depth.r - 1.0;
		float pixelDepth = 2.0 * uNearPlane * uFarPlane / (uFarPlane + uNearPlane - z_n * (uFarPlane - uNearPlane));

		vec4 sum = vec4(0,0,0,1);
        vec2 magnitude = uBlurAmount / (uScreenSize/vec2(1280.0,720.0));
		
		for(int i = 0; i < 2; i++)
		{
			for(int q = 0; q < 6; q++)
			{
				float offset = 5.0 - float(q);
				float kernel = 0.01;
				vec2 direction = uBlurDirection;
				
				if(i==1)
					direction *= -1.0;
				
				if(q==0)
					kernel = 0.01;
				else if(q==1)
					kernel = 0.05;
				else if(q==2)
					kernel = 0.09;
				else if(q==3)
					kernel = 0.12;
				else if(q==4)
					kernel = 0.15;
				else if(q==5)
					kernel = 0.16;
				
				vec2 uv = vec2(vTextureCoord.x - (offset * magnitude.x * direction.x), vTextureCoord.y - (offset * magnitude.y * direction.y));
				
					depth = texture2D(uDepthSampler, uv);
					z_n = 2.0 * depth.r - 1.0;
					float sampledDepth = 2.0 * uNearPlane * uFarPlane / (uFarPlane + uNearPlane - z_n * (uFarPlane - uNearPlane));
					
					if(q == 5 || 
					  ((sampledDepth < uFocusEnd || pixelDepth < uFocusEnd) && abs(sampledDepth - pixelDepth) > uDepthBias) &&
					  !(sampledDepth > uFocusEnd && pixelDepth < uFocusStart))
					{
						sum += baseSample * kernel;
					}
					else
						sum += texture2D(uDiffuseSampler, uv) * kernel;
		 
				if(i==1 && q == 4)
					break;
			}
		}

        gl_FragColor = sum * vec4(1,1,1,uAlphaMultiplier);
}