#version 150

#define PI  3.14159265

varying vec2 vTextureCoord;
varying vec4 vPosition;

uniform sampler2D uDiffuseSampler;
uniform sampler2D uDepthSampler;
uniform vec2 uScreenSize;

vec2 pixel = vec2(1.0) / uScreenSize;
vec2 screenSizeMagnitudeOffset = uScreenSize / vec2(1280.0,720.0);

uniform float uNdofstart;
uniform float uFdofstart;
uniform float uFalloff; 

uniform float uNearPlane;
uniform float uFarPlane;

uniform float uMaxBlur;
uniform float uThreshold;
uniform float uGain;
uniform float uBias;

int samples = 5;
int rings = 5;

float namount = 0.0001;

vec3 color(vec2 coords,float blur)
{
	vec3 col = vec3(0.0);
	
	col.r = texture2D(uDiffuseSampler,coords + vec2(0.0,1.0)*pixel*blur).r;
	col.g = texture2D(uDiffuseSampler,coords + vec2(-0.866,-0.5)*pixel*blur).g;
	col.b = texture2D(uDiffuseSampler,coords + vec2(0.866,-0.5)*pixel*blur).b;
	
	vec3 lumcoeff = vec3(0.299,0.587,0.114);
	float lum = dot(col.rgb, lumcoeff);
	float thresh = max((lum-uThreshold)*uGain, 0.0);
	return col+mix(vec3(0.0),col,thresh*blur);
}

vec2 rand(vec2 coord)
{
	float noiseX = ((fract(1.0-coord.s*(uScreenSize.x/2.0))*0.25)+(fract(coord.t*(uScreenSize.y/2.0))*0.75))*2.0-1.0;
	float noiseY = ((fract(1.0-coord.s*(uScreenSize.x/2.0))*0.75)+(fract(coord.t*(uScreenSize.y/2.0))*0.25))*2.0-1.0;
	noiseX = clamp(fract(sin(dot(coord ,vec2(12.9898,78.233))) * 43758.5453),0.0,1.0)*2.0-1.0;
	noiseY = clamp(fract(sin(dot(coord ,vec2(12.9898,78.233)*2.0)) * 43758.5453),0.0,1.0)*2.0-1.0;
	
	return vec2(noiseX,noiseY);
}

float linearize(float depth)
{
	return -uFarPlane * uNearPlane / (depth * (uFarPlane - uNearPlane) - uFarPlane);
}

bool shouldSample(vec2 uv, float referenceDepth)
{
	// Near blur
	if(referenceDepth <= uNdofstart)
	{
		// Since it's near blur, blur in any sample around it
		return true;
	}
	// Far blur
	else if(referenceDepth >= uFdofstart)
	{
		float sampleDepth = linearize(texture2D(uDepthSampler,uv).x);
		
		if(sampleDepth > uNdofstart && sampleDepth < uFdofstart)
			return false;
			
		return true;
	}
	// Then it must be in focal range, don't blur in samples from other planes.
	// Probably won't get called due to early-out
	else
	{
		return false;
	}
}

void main() 
{
	float depth = linearize(texture2D(uDepthSampler,vTextureCoord).x);
	float fDepth = (uNdofstart + uFdofstart) * 0.5;
	
	float blur = 0.0;
	
	if(depth > uNdofstart && depth < uFdofstart)
	{
		blur = 0.0;
	}
	else
	{
		if(depth < uNdofstart)
		{
			blur = 1.0 - ((depth - (uNdofstart - uFalloff)) / uFalloff);
		}
		else
		{
			blur = (depth - uFdofstart) / uFalloff;
		}
	}
	
	blur = clamp(blur,0.0,1.0);
	vec2 noise = rand(vTextureCoord)*namount*blur;
	float w = pixel.x*blur*screenSizeMagnitudeOffset.x*uMaxBlur+noise.x;
	float h = pixel.y*blur*screenSizeMagnitudeOffset.y*uMaxBlur+noise.y;
	
	vec3 col = vec3(0.0);
	
	if(blur < 0.05)
	{
		col = texture2D(uDiffuseSampler, vTextureCoord).rgb;
	}
	else
	{
		col = texture2D(uDiffuseSampler, vTextureCoord).rgb;
		float s = 1.0;
		int ringsamples;
		
		for (int i = 1; i <= rings; i += 1)
		{   
			ringsamples = i * samples;
			
			for (int j = 0 ; j < ringsamples ; j += 1)   
			{
				float step = PI*2.0 / float(ringsamples);
				float pw = (cos(float(j)*step)*float(i));
				float ph = (sin(float(j)*step)*float(i));
				
				vec2 uv = vTextureCoord + vec2(pw*w,ph*h);
				
				if(shouldSample(uv, depth))
				{
					col += color(uv,blur)*mix(1.0,(float(i))/(float(rings)),uBias);  
					s += 1.0*mix(1.0,(float(i))/(float(rings)),uBias);   
				}
			}
		}
		
		col /= s;
	}
	
	gl_FragColor.rgb = col;
	gl_FragColor.a = 1.0;
}