#version 150

varying vec2 vTextureCoord;
varying vec4 vPosition;

uniform sampler2D uBaseSampler;
uniform sampler2D uDiffuseSampler;
uniform sampler2D uDepthSampler;
uniform float uAlphaMultiplier;

uniform float uFocusStart;
uniform float uFocusEnd;
uniform float uFalloff;
uniform float uNearPlane;
uniform float uFarPlane;

void main(void)
{
    vec4 depth = texture2D(uDepthSampler, vec2(vTextureCoord.s, vTextureCoord.t));
    float z_n = 2.0 * depth.r - 1.0;
    float linearDepth = 2.0 * uNearPlane * uFarPlane / (uFarPlane + uNearPlane - z_n * (uFarPlane - uNearPlane));

    vec4 sum = vec4(0.0);

    if(linearDepth >= uFocusStart && linearDepth <= uFocusEnd)
    {
       gl_FragColor = texture2D(uBaseSampler, vTextureCoord) * vec4(1,1,1,uAlphaMultiplier);
    }
    else
    {
        float fallMod = 1.0;

        if(linearDepth < uFocusStart)
        {
           fallMod = min(1.0, 1.0 - ((linearDepth - (uFocusStart - uFalloff)) / uFalloff));
        }
        else
        {
           fallMod = min(1.0, ((linearDepth - uFocusEnd) / uFalloff));
        }
		
		vec4 aSample = texture2D(uDiffuseSampler, vTextureCoord);
		vec4 bSample = texture2D(uBaseSampler, vTextureCoord);
		
		sum = bSample + (aSample - bSample) * fallMod;

        gl_FragColor = sum * vec4(1,1,1,uAlphaMultiplier);
    }
}