#version 150


varying vec3 vTransformedNormal;
varying vec4 vPosition;
varying vec2 vTextureCoord;
varying vec4 vTangent;
uniform sampler2D uDiffuseSampler;
uniform sampler2D uDisplaceSampler;
uniform float uAlphaMultiplier;
uniform vec4 uDiffuseValue;
uniform vec3 uAmbientValue;
uniform vec3 uPointLightingPositions[10]; 
uniform vec3 uPointLightingColors[10];
uniform float uPointLightingInvRadii[10];
uniform float uPointLightDistanceOffset[10];
uniform int uPointLightCount;
uniform vec3 uDirectionalLightingDirections[10];
uniform vec3 uDirectionalLightingColors[10];
uniform int uDirectionalLightCount;
uniform vec3 uSpotLightingPositions[10];
uniform vec3 uSpotLightingDirections[10];
uniform vec3 uSpotLightingColors[10];
uniform float uSpotLightingInvRadii[10];
uniform float uSpotLightingCutoffs[10];
uniform float uSpotLightingOuterCutoffs[10];
uniform int uSpotLightCount;
uniform vec3 uCameraPosition;
uniform vec4 uFogColor;
uniform float uFogDensity;
uniform float uFogHeightStart;
uniform float uFogHeightFalloff;
uniform samplerCube uEnvironmentSampler;
uniform float uEnvironmentStrength;

uniform float uMagnitude;
uniform vec2 uScreenSize;

void main(void)
{
	vec3 normal_3;
	vec3 normal = normalize(vTransformedNormal);
  vec3 tangent = normalize(vTangent.xyz);
  vec3 tmpvar_6 = ((normal.yzx * tangent.zxy) - (normal.zxy * tangent.yzx));
  mat3 tmpvar_7;
  tmpvar_7[0].x = tangent.x;
  tmpvar_7[0].y = tmpvar_6.x;
  tmpvar_7[0].z = normal.x;
  tmpvar_7[1].x = tangent.y;
  tmpvar_7[1].y = tmpvar_6.y;
  tmpvar_7[1].z = normal.y;
  tmpvar_7[2].x = tangent.z;
  tmpvar_7[2].y = tmpvar_6.z;
  tmpvar_7[2].z = normal.z;
  normal = normalize((normalize(
    ((texture2D (uDisplaceSampler, vTextureCoord).xyz * 2.0) - 1.0)
  ) * tmpvar_7));

    vec2 tC = vTextureCoord.st;
    vec2 displace = normal.rg;

    displace *= uMagnitude;
    displace /= uScreenSize;
vec4 background = texture2D(uDiffuseSampler, (gl_FragCoord.xy / uScreenSize) + displace);

vec3 eyeDirection = normalize(uCameraPosition - vPosition.xyz);
vec3 lightWeighting = vec3(0,0,0);
for(int i = 0; i < 10; i++)
{
float att = 0.0;
if(i < uPointLightCount)
{
vec3 pointLightDirection = normalize(uPointLightingPositions[i] - vPosition.xyz);
float pointLightDistance = length(uPointLightingPositions[i] - vPosition.xyz)-uPointLightDistanceOffset[i]; 
att = clamp(1.0 - uPointLightingInvRadii[i] * sqrt(dot(pointLightDistance, pointLightDistance)), 0.0, 1.0);
float pointLightWeighting = max(dot(normal, pointLightDirection), 0.0);
lightWeighting += uPointLightingColors[i] * pointLightWeighting * att;
}
if(i < uDirectionalLightCount)
{
float directionalLightWeighting = max(dot(normal, uDirectionalLightingDirections[i]), 0.0);
lightWeighting += (uDirectionalLightingColors[i] * directionalLightWeighting);
}
if(i < uSpotLightCount)
{
vec3 directionToSpot = (uSpotLightingPositions[i] - vPosition.xyz);
att = clamp(1.0 - uSpotLightingInvRadii[i] * sqrt(dot(directionToSpot, directionToSpot)), 0.0, 1.0);
directionToSpot = normalize(directionToSpot);
vec3 spotDirection = normalize(uSpotLightingDirections[i]);
float spotLightWeighting = max(dot(normal, directionToSpot), 0.0);
float spotAngle = dot(-directionToSpot, spotDirection);
if(spotLightWeighting > 0.0)
{
vec3 spotContribution = vec3(0,0,0);
float innerCutoffCosine = cos(uSpotLightingCutoffs[i]);
float outerCutoffCosine = cos(uSpotLightingOuterCutoffs[i]);
if( spotAngle > innerCutoffCosine )
{
spotContribution = uSpotLightingColors[i] * spotLightWeighting * att;
}
else if( spotAngle > outerCutoffCosine )
{
float falloff = (spotAngle - outerCutoffCosine) / (innerCutoffCosine - outerCutoffCosine);
spotContribution = uSpotLightingColors[i] * spotLightWeighting * falloff * att;
}
lightWeighting += spotContribution;
}
}
}
lightWeighting += uAmbientValue;

gl_FragColor = vec4(uDiffuseValue.rgb * lightWeighting, 1.0);
gl_FragColor = mix(background, gl_FragColor, uDiffuseValue.a);
gl_FragColor.a = uAlphaMultiplier;


vec3 envLookup = normalize(reflect(eyeDirection, vTransformedNormal));
float envStrength = uEnvironmentStrength;
envLookup.z *= -1.0;
gl_FragColor.rgb += texture(uEnvironmentSampler, envLookup).rgb * envStrength;

	if (vPosition.y <= uFogHeightStart)
	{
		const float LOG2 = 1.442695;
		float z = gl_FragCoord.z / gl_FragCoord.w;
		float fogFactor = exp2(-uFogDensity * uFogDensity * z * z * LOG2) / clamp(((vPosition.y - uFogHeightStart) / uFogHeightFalloff), 0.0, 1.0); 
		fogFactor = clamp(fogFactor, 0.0, 1.0);
		gl_FragColor = mix(uFogColor, gl_FragColor, fogFactor);
	}
}
