#version 150


varying vec3 vTransformedNormal;
varying vec4 vPosition;
uniform vec4 uDiffuseValue;
uniform vec3 uAmbientValue;
uniform vec3 uPointLightingPositions[15];
uniform vec3 uPointLightingColors[15];
uniform float uPointLightingInvRadii[15];
uniform int uPointLightCount;
uniform vec3 uDirectionalLightingDirections[15];
uniform vec3 uDirectionalLightingColors[15];
uniform int uDirectionalLightCount;
uniform vec3 uSpotLightingPositions[15];
uniform vec3 uSpotLightingDirections[15];
uniform vec3 uSpotLightingColors[15];
uniform float uSpotLightingInvRadii[15];
uniform float uSpotLightingCutoffs[15];
uniform float uSpotLightingOuterCutoffs[15];
uniform int uSpotLightCount;
uniform vec4 uFogColor;
uniform float uFogDensity;

uniform float uFadeLimit;
uniform vec3 uCameraLook;
uniform vec3 uCameraDirection;

void main(void)
{
vec3 normal = normalize(vTransformedNormal);
vec3 lightWeighting = vec3(0,0,0);
for(int i = 0; i < 15; i++)
{
float att = 0.0;
if(i < uPointLightCount)
{
vec3 pointLightDirection = (uPointLightingPositions[i] - vPosition.xyz);
att = clamp(1.0 - uPointLightingInvRadii[i] * sqrt(dot(pointLightDirection, pointLightDirection)), 0.0, 1.0);
pointLightDirection = normalize(pointLightDirection);
float pointLightWeighting = max(dot(normal, pointLightDirection), 0.0);
lightWeighting += uPointLightingColors[i] * pointLightWeighting * att;
}
if(i < uDirectionalLightCount)
{
float directionalLightWeighting = max(dot(normal, uDirectionalLightingDirections[i]), 0.0);
lightWeighting += (uDirectionalLightingColors[i] * directionalLightWeighting);
}
if(i < uSpotLightCount)
{
vec3 directionToSpot = (uSpotLightingPositions[i] - vPosition.xyz);
att = clamp(1.0 - uSpotLightingInvRadii[i] * sqrt(dot(directionToSpot, directionToSpot)), 0.0, 1.0);
directionToSpot = normalize(directionToSpot);
vec3 spotDirection = normalize(uSpotLightingDirections[i]);
float spotLightWeighting = max(dot(normal, directionToSpot), 0.0);
float spotAngle = dot(-directionToSpot, spotDirection);
if(spotLightWeighting > 0.0)
{
vec3 spotContribution = vec3(0,0,0);
float innerCutoffCosine = cos(uSpotLightingCutoffs[i]);
float outerCutoffCosine = cos(uSpotLightingOuterCutoffs[i]);
if( spotAngle > innerCutoffCosine )
{
spotContribution = uSpotLightingColors[i] * spotLightWeighting * att;
}
else if( spotAngle > outerCutoffCosine )
{
float falloff = (spotAngle - outerCutoffCosine) / (innerCutoffCosine - outerCutoffCosine);
spotContribution = uSpotLightingColors[i] * spotLightWeighting * falloff * att;
}
lightWeighting += spotContribution;
}
}
}
lightWeighting += uAmbientValue;
gl_FragColor = vec4(uDiffuseValue.rgb * lightWeighting, uDiffuseValue.a);
const float LOG2 = 1.442695;
float z = gl_FragCoord.z / gl_FragCoord.w;
float fogFactor = exp2(-uFogDensity * uFogDensity * z * z * LOG2);
fogFactor = clamp(fogFactor, 0.0, 1.0);


vec3 toLook = normalize(uCameraLook.xyz - vPosition.xyz);
vec3 cameraDir = -normalize(uCameraDirection);
float d = 1.0-dot(toLook, cameraDir);

if(d < 0.25)
{
	gl_FragColor.a *= pow(d/0.25, 8.0);

    if(gl_FragColor.a < 0.1)
    discard;
}

gl_FragColor = mix(uFogColor, gl_FragColor, fogFactor);

}
