#version 150

varying vec3 vTransformedNormal;
varying vec4 vPosition;
varying vec2 vTextureCoord;

uniform vec2 uTexCoordOffset;

uniform sampler2D uDiffuseSampler;
uniform vec4 uDiffuseValue;
uniform float uAlphaMultiplier;

uniform vec3 uAmbientValue;
uniform vec3 uPointLightingPositions[15];
uniform vec3 uPointLightingColors[15];
uniform float uPointLightingInvRadii[15];
uniform float uPointLightDistanceOffset[10];
uniform int uPointLightCount;
uniform vec3 uDirectionalLightingDirections[15];
uniform vec3 uDirectionalLightingColors[15];
uniform int uDirectionalLightCount;
uniform vec3 uSpotLightingPositions[15];
uniform vec3 uSpotLightingDirections[15];
uniform vec3 uSpotLightingColors[15];
uniform float uSpotLightingInvRadii[15];
uniform float uSpotLightingCutoffs[15];
uniform float uSpotLightingOuterCutoffs[15];
uniform int uSpotLightCount;
uniform vec4 uFogColor;
uniform float uFogDensity;

void main(void)
{ 
	vec3 normal = normalize(vTransformedNormal);
vec3 lightWeighting = vec3(0,0,0);
for(int i = 0; i < 15; i++)
{
float att = 0.0;
if(i < uPointLightCount)
{
vec3 pointLightDirection = normalize(uPointLightingPositions[i] - vPosition.xyz);

vec3 pointLightDistance = (uPointLightingPositions[i] - vPosition.xyz) - uPointLightDistanceOffset[i];
att = clamp(1.0 - uPointLightingInvRadii[i] * sqrt(dot(pointLightDistance, pointLightDistance)), 0.0, 1.0);

float pointLightWeighting = max(dot(normal, pointLightDirection), 0.0);
lightWeighting += uPointLightingColors[i] * pointLightWeighting * att;
}
if(i < uDirectionalLightCount)
{
float directionalLightWeighting = max(dot(normal, uDirectionalLightingDirections[i]), 0.0);
lightWeighting += (uDirectionalLightingColors[i] * directionalLightWeighting);
}
if(i < uSpotLightCount)
{
vec3 directionToSpot = (uSpotLightingPositions[i] - vPosition.xyz);
att = clamp(1.0 - uSpotLightingInvRadii[i] * sqrt(dot(directionToSpot, directionToSpot)), 0.0, 1.0);
directionToSpot = normalize(directionToSpot);
vec3 spotDirection = normalize(uSpotLightingDirections[i]);
float spotLightWeighting = max(dot(normal, directionToSpot), 0.0);
float spotAngle = dot(-directionToSpot, spotDirection);
if(spotLightWeighting > 0.0)
{
vec3 spotContribution = vec3(0,0,0);
float innerCutoffCosine = cos(uSpotLightingCutoffs[i]);
float outerCutoffCosine = cos(uSpotLightingOuterCutoffs[i]);
if( spotAngle > innerCutoffCosine )
{
spotContribution = uSpotLightingColors[i] * spotLightWeighting * att;
}
else if( spotAngle > outerCutoffCosine )
{
float falloff = (spotAngle - outerCutoffCosine) / (innerCutoffCosine - outerCutoffCosine);
spotContribution = uSpotLightingColors[i] * spotLightWeighting * falloff * att;
}
lightWeighting += spotContribution;
}
}
}
lightWeighting += uAmbientValue;
vec4 textureValue = texture2D(uDiffuseSampler, vTextureCoord + uTexCoordOffset).r * uDiffuseValue;

gl_FragColor = vec4(textureValue.rgb * lightWeighting, uAlphaMultiplier);

const float LOG2 = 1.442695;
float z = gl_FragCoord.z / gl_FragCoord.w;
float fogFactor = exp2(-uFogDensity * uFogDensity * z * z * LOG2);
fogFactor = clamp(fogFactor, 0.0, 1.0);
gl_FragColor = mix(uFogColor, gl_FragColor, fogFactor);
	
}