#version 150

varying vec2 vTextureCoord;
varying vec3 vTransformedNormal;
varying vec4 vPosition;

uniform vec3 uAmbientValue;

uniform vec3 uPointLightingPositions[5];
uniform vec3 uPointLightingColors[5];
uniform float uPointLightingInvRadii[5];

uniform vec3 uDirectionalLightingDirections[5];
uniform vec3 uDirectionalLightingColors[5];

uniform vec3 uSpotLightingPositions[5];
uniform vec3 uSpotLightingDirections[5];
uniform vec3 uSpotLightingColors[5];
uniform float uSpotLightingInvRadii[5];
uniform float uSpotLightingCutoffs[5];
uniform float uSpotLightingOuterCutoffs[5];

uniform vec3 uCameraPosition;

uniform sampler2D uDiffuseSampler;
uniform sampler2D uBaseSampler;

void main(void)
{
    vec3 normal = normalize(vTransformedNormal);
    vec3 lightWeighting = uAmbientValue;
    vec3 eyeDirection = normalize(uCameraPosition - vPosition.xyz);

    for(int i = 0; i < 5; i++)
        {
            vec3 pointLightDirection = (uPointLightingPositions[i] - vPosition.xyz);
            float att = clamp(1.0 - uPointLightingInvRadii[i] * sqrt(dot(pointLightDirection, pointLightDirection)), 0.0, 1.0);
            pointLightDirection = normalize(pointLightDirection);
            float pointLightWeighting = max(dot(normal, pointLightDirection), 0.0);
            lightWeighting += uPointLightingColors[i] * pointLightWeighting * att;

            float directionalLightWeighting = max(dot(normal, uDirectionalLightingDirections[i]), 0.0);
            lightWeighting += (uDirectionalLightingColors[i] * directionalLightWeighting);

                                    vec3 directionToSpot = (uSpotLightingPositions[i] - vPosition.xyz);
                                    att = clamp(1.0 - uSpotLightingInvRadii[i] * sqrt(dot(directionToSpot, directionToSpot)), 0.0, 1.0);
                                    directionToSpot = normalize(directionToSpot);
                                    vec3 spotDirection = normalize(uSpotLightingDirections[i]);
                                    float spotLightWeighting = max(dot(normal, directionToSpot), 0.0);
                                    float spotAngle = dot(-directionToSpot, spotDirection);

                                    if(spotLightWeighting > 0.0)
                                    {
                                        vec3 spotContribution = vec3(0,0,0);
                                        float innerCutoffCosine = cos(uSpotLightingCutoffs[i]);
                                        float outerCutoffCosine = cos(uSpotLightingOuterCutoffs[i]);

                                        if( spotAngle > innerCutoffCosine )
                                        {
                                            spotContribution = uSpotLightingColors[i] * spotLightWeighting * att;
                                        }
                                        else if( spotAngle > outerCutoffCosine )
                                        {
                                            float falloff = (spotAngle - outerCutoffCosine) / (innerCutoffCosine - outerCutoffCosine);
                                            spotContribution = uSpotLightingColors[i] * spotLightWeighting * falloff * att;
                                        }

                                        lightWeighting += spotContribution;
                                     }
        }

    vec4 textureColor = texture2D(uDiffuseSampler, vec2(vTextureCoord.s, vTextureCoord.t));

    gl_FragColor = vec4(textureColor.rgb * lightWeighting, textureColor.a);
}