#version 150

varying vec2 vTextureCoord;
varying vec4 vPosition;
varying vec3 vTransformedNormal;

uniform vec4 uDiffuseValue;
uniform vec3 uSpecularValue;
uniform float uSpecularIntensity;
uniform sampler2D uDiffuseSampler;
uniform float uAlphaMultiplier;

uniform float uMagnitude;
uniform vec2 uScreenSize;

uniform vec3 uAmbientValue;
uniform vec3 uPointLightingPositions[10];
uniform vec3 uPointLightingColors[10];
uniform float uPointLightingInvRadii[10];
uniform float uPointLightDistanceOffset[10];
uniform int uPointLightCount;
uniform vec3 uDirectionalLightingDirections[10];
uniform vec3 uDirectionalLightingColors[10];
uniform int uDirectionalLightCount;
uniform vec3 uSpotLightingPositions[10];
uniform vec3 uSpotLightingDirections[10];
uniform vec3 uSpotLightingColors[10];
uniform float uSpotLightingInvRadii[10];
uniform float uSpotLightingCutoffs[10];
uniform float uSpotLightingOuterCutoffs[10];
uniform int uSpotLightCount;
uniform vec4 uFogColor;
uniform float uFogDensity;
uniform float uFogHeightStart;
uniform float uFogHeightFalloff;
uniform vec3 uCameraPosition;

uniform sampler2D uDepthSampler;
uniform float uNearPlane;
uniform float uFarPlane;
uniform float uDepthFalloff;
uniform float uDepthBias;
uniform float uDepthFalloffBrightness;

uniform samplerCube uEnvironmentSampler;
uniform float uEnvironmentStrength;

float linearize(float depth)
{
	return 2.0 * uNearPlane * uFarPlane / (uFarPlane + uNearPlane - depth * (uFarPlane - uNearPlane));
}

void main(void)
{
    vec2 displace = vTransformedNormal.xz;

    displace *= uMagnitude;
    displace /= uScreenSize;
	
	vec4 background = texture2D(uDiffuseSampler, (gl_FragCoord.xy / uScreenSize) + displace);
	background.rgb *= uDiffuseValue.rgb;
	
	background.rgb = mix(background.rgb, uDiffuseValue.rgb, uDiffuseValue.a);
	
    vec3 lightWeighting_2;
  vec3 specularHighlight_3;
  vec3 specularValue_4;
  float specularIntensity_5;
  vec3 eyeDirection_6;
  vec3 normal_7;
  normal_7 = normalize(vTransformedNormal);
  eyeDirection_6 = normalize((uCameraPosition - vPosition.xyz));
  specularIntensity_5 = uSpecularIntensity;
  specularValue_4 = uSpecularValue;
  specularHighlight_3 = vec3(0.0, 0.0, 0.0);
  lightWeighting_2 = vec3(0.0, 0.0, 0.0);
   for (int i_1 = 0; i_1 < 10; i_1++) {
    if ((i_1 < uPointLightCount)) {
      vec3 tmpvar_8;
      tmpvar_8 = normalize((uPointLightingPositions[i_1] - vPosition.xyz));
      vec3 x_9;
      x_9 = (uPointLightingPositions[i_1] - vPosition.xyz);
      float tmpvar_10;
      tmpvar_10 = (sqrt(dot (x_9, x_9)) - uPointLightDistanceOffset[i_1]);
      float tmpvar_11;
      tmpvar_11 = clamp ((1.0 - (uPointLightingInvRadii[i_1] * 
        sqrt((tmpvar_10 * tmpvar_10))
      )), 0.0, 1.0);
      lightWeighting_2 = (lightWeighting_2 + ((uPointLightingColors[i_1] * 
        max (dot (normal_7, tmpvar_8), 0.0)
      ) * tmpvar_11));
      vec3 I_12;
      I_12 = -(tmpvar_8);
      specularHighlight_3 = (specularHighlight_3 + ((
        (specularValue_4 * uPointLightingColors[i_1])
       * 
        pow (max (dot ((I_12 - 
          (2.0 * (dot (normal_7, I_12) * normal_7))
        ), eyeDirection_6), 0.0), specularIntensity_5)
      ) * tmpvar_11));
    };
    if ((i_1 < uDirectionalLightCount)) {
      lightWeighting_2 = (lightWeighting_2 + (uDirectionalLightingColors[i_1] * max (
        dot (normal_7, uDirectionalLightingDirections[i_1])
      , 0.0)));
    };
    if ((i_1 < uSpotLightCount)) {
      vec3 tmpvar_13;
      tmpvar_13 = (uSpotLightingPositions[i_1] - vPosition.xyz);
      float tmpvar_14;
      tmpvar_14 = clamp ((1.0 - (uSpotLightingInvRadii[i_1] * 
        sqrt(dot (tmpvar_13, tmpvar_13))
      )), 0.0, 1.0);
      vec3 tmpvar_15;
      tmpvar_15 = normalize(tmpvar_13);
      float tmpvar_16;
      tmpvar_16 = max (dot (normal_7, tmpvar_15), 0.0);
      float tmpvar_17;
      tmpvar_17 = dot (-(tmpvar_15), normalize(uSpotLightingDirections[i_1]));
      if ((tmpvar_16 > 0.0)) {
        vec3 spotContribution_18;
        spotContribution_18 = vec3(0.0, 0.0, 0.0);
        float tmpvar_19;
        tmpvar_19 = cos(uSpotLightingCutoffs[i_1]);
        float tmpvar_20;
        tmpvar_20 = cos(uSpotLightingOuterCutoffs[i_1]);
        if ((tmpvar_17 > tmpvar_19)) {
          spotContribution_18 = ((uSpotLightingColors[i_1] * tmpvar_16) * tmpvar_14);
        } else {
          if ((tmpvar_17 > tmpvar_20)) {
            spotContribution_18 = (((uSpotLightingColors[i_1] * tmpvar_16) * (
              (tmpvar_17 - tmpvar_20)
             / 
              (tmpvar_19 - tmpvar_20)
            )) * tmpvar_14);
          };
        };
        lightWeighting_2 = (lightWeighting_2 + spotContribution_18);
      };
    };
  };
  lightWeighting_2 = (lightWeighting_2 + uAmbientValue);
  vec4 tmpvar_20;
  tmpvar_20.xyz = (background.rgb * lightWeighting_2);
  tmpvar_20.w = uAlphaMultiplier;
  gl_FragColor.w = tmpvar_20.w;
  gl_FragColor.xyz = (tmpvar_20.xyz + specularHighlight_3);
  
vec3 envLookup = normalize(reflect(eyeDirection_6, normal_7));
float envStrength = uEnvironmentStrength;
envLookup.z *= -1.0;
gl_FragColor.rgb += texture(uEnvironmentSampler, envLookup).rgb * envStrength;
  
  if(vPosition.y <= uFogHeightStart)
	{
  float tmpvar_21;
  tmpvar_21 = (gl_FragCoord.z / gl_FragCoord.w);
  gl_FragColor = mix (uFogColor, gl_FragColor, clamp ( exp2(
    ((((
      -(uFogDensity)
     * uFogDensity) * tmpvar_21) * tmpvar_21) * 1.442695)
  ) / clamp(((vPosition.y - uFogHeightStart) / uFogHeightFalloff), 0.0, 1.0), 0.0, 1.0));
  }
  
    float depth = texture2D(uDepthSampler, (gl_FragCoord.xy / uScreenSize)).r;
    float linearWorldDepth = 2.0 * uNearPlane * uFarPlane / (uFarPlane + uNearPlane - depth * (uFarPlane - uNearPlane));
    float fragDepth = 2.0 * uNearPlane * uFarPlane / (uFarPlane + uNearPlane - gl_FragCoord.z * (uFarPlane - uNearPlane));
	float difference = (linearWorldDepth - fragDepth);
	
	if(difference > 0.0)
	{
		if(difference < uDepthFalloff * (0.5 + uDepthBias))
		{
			gl_FragColor.rgb = vec3(uDepthFalloffBrightness);
			
			gl_FragColor.a = mix(gl_FragColor.a, 0.0, 1.0-(difference / uDepthFalloff));
		}
		else if(difference < uDepthFalloff)
		{
			float falloff = uDepthFalloff * (0.5 - uDepthBias);
			float diff = difference - falloff;
			gl_FragColor.rgb = mix(gl_FragColor.rgb, vec3(uDepthFalloffBrightness), 1.0 - (diff / falloff));
			
			gl_FragColor.a = mix(gl_FragColor.a, 0.0, 1.0-(difference / uDepthFalloff));
		}
	}
}