#version 150

attribute vec3 aVertexPosition;
attribute vec2 aTextureCoord;
attribute vec4 aTangent;

uniform mat4 uModelMatrix;
uniform mat4 uViewMatrix;
uniform mat4 uProjectionMatrix;
uniform mat3 uNormalMatrix;

varying vec2 vTextureCoord;
varying vec3 vTransformedNormal;
varying vec4 vPosition;
varying float vOffset;

uniform float uSineOffsetX;
uniform float uSineOffsetY;
uniform float uOffsetX;
uniform float uOffsetY;
uniform float uSineWavelengthX;
uniform float uSineWavelengthY;
uniform float uVerticalMagnitude;

void main(void)
{
    vec3 n = vec3(0,1,0);
    vec3 t = vec3(-1,0,0);
    vec3 b = vec3(0,0,-1);

    vec3 pointA = aVertexPosition + (t*0.1);
    vec3 pointB = aVertexPosition + (b*0.1);

    pointA.y += (sin((pointA.x + uSineOffsetX) * uSineWavelengthX) + sin((pointA.z + uSineOffsetY) * uSineWavelengthY)) * uVerticalMagnitude;
    pointB.y += (sin((pointB.x + uSineOffsetX) * uSineWavelengthX) + sin((pointB.z + uSineOffsetY) * uSineWavelengthY)) * uVerticalMagnitude;

     vec3 u = normalize(aVertexPosition-pointA);
     vec3 v = normalize(aVertexPosition-pointB);

     vec3 normal = -normalize(cross(u, v));

    vOffset = (sin((aVertexPosition.x + uSineOffsetX) * uSineWavelengthX) + sin((aVertexPosition.z + uSineOffsetY) * uSineWavelengthY)) * uVerticalMagnitude;

    vPosition = vec4(aVertexPosition, 1.0);
    vPosition.y += vOffset;
    vPosition = uModelMatrix * vPosition;

    gl_Position = uProjectionMatrix * uViewMatrix * vPosition;
    vTextureCoord = aTextureCoord + vec2(uOffsetX, uOffsetY);
    vTransformedNormal = uNormalMatrix * normal;
}