#version 150

varying vec2 vTextureCoord;
varying vec4 vPosition;
varying vec4 vTangent;
varying vec3 vTransformedNormal;

uniform sampler2D uDiffuseSampler;
uniform vec4 uDiffuseValue;
uniform sampler2D uDisplaceSampler;
uniform float uAlphaMultiplier;

uniform float uMagnitude;
uniform vec2 uScreenSize;

uniform vec3 uSpecularValue;
//uniform vec3 uAmbientValue;
uniform float uSpecularIntensity;
uniform vec3 uPointLightingPositions[10];
uniform vec3 uPointLightingColors[10];
uniform float uPointLightingInvRadii[10];
uniform int uPointLightCount;
uniform vec3 uDirectionalLightingDirections[10];
uniform vec3 uDirectionalLightingColors[10];
uniform int uDirectionalLightCount;
uniform vec3 uSpotLightingPositions[10];
uniform vec3 uSpotLightingDirections[10];
uniform vec3 uSpotLightingColors[10];
uniform float uSpotLightingInvRadii[10];
uniform float uSpotLightingCutoffs[10];
uniform float uSpotLightingOuterCutoffs[10];
uniform int uSpotLightCount;
uniform vec3 uCameraPosition;
uniform vec4 uFogColor;
uniform float uFogDensity;

uniform float uWorldTopCutoff;
uniform float uWorldBottomCutoff;
uniform float uWorldHeightCutoffSpread;

void main(void)
{
	if( vPosition.y > uWorldTopCutoff &&
		vPosition.y < uWorldBottomCutoff )
	{
		discard;
	}

  vec3 normal_3;
  vec3 tmpvar_4;
  tmpvar_4 = normalize(vTransformedNormal);
  vec3 tmpvar_5;
  tmpvar_5 = normalize(vTangent.xyz);
  vec3 tmpvar_6;
  tmpvar_6 = ((tmpvar_4.yzx * tmpvar_5.zxy) - (tmpvar_4.zxy * tmpvar_5.yzx));
  mat3 tmpvar_7;
  tmpvar_7[0].x = tmpvar_5.x;
  tmpvar_7[0].y = tmpvar_6.x;
  tmpvar_7[0].z = tmpvar_4.x;
  tmpvar_7[1].x = tmpvar_5.y;
  tmpvar_7[1].y = tmpvar_6.y;
  tmpvar_7[1].z = tmpvar_4.y;
  tmpvar_7[2].x = tmpvar_5.z;
  tmpvar_7[2].y = tmpvar_6.z;
  tmpvar_7[2].z = tmpvar_4.z;
  normal_3 = normalize((normalize(
    ((texture2D (uDisplaceSampler, vTextureCoord).xyz * 2.0) - 1.0)
  ) * tmpvar_7));

    vec2 tC = vTextureCoord.st;
    vec2 displace = normal_3.rg;

    displace *= uMagnitude;
    displace /= uScreenSize;
	
	vec4 background = texture2D(uDiffuseSampler, (gl_FragCoord.xy / uScreenSize) + displace);
	background.rgb *= uDiffuseValue.rgb;
	
	vec4 frag = mix(background, uDiffuseValue, uDiffuseValue.a);
	frag.a *= uAlphaMultiplier;
	
vec3 lightWeighting_2;
  vec3 specularHighlight_3;
  vec3 specularValue_4;
  float specularIntensity_5;
  vec3 eyeDirection_6;
  vec3 normal_7;
  normal_7 = normalize(normal_3);
  eyeDirection_6 = normalize((uCameraPosition - vPosition.xyz));
  specularIntensity_5 = uSpecularIntensity;
  specularValue_4 = uSpecularValue;
  specularHighlight_3 = vec3(0.0, 0.0, 0.0);
  lightWeighting_2 = vec3(0.0, 0.0, 0.0);
  for (int i_1 = 0; i_1 < 10; i_1++) {
    if ((i_1 < uPointLightCount)) {
      vec3 tmpvar_8;
      tmpvar_8 = (uPointLightingPositions[i_1] - vPosition.xyz);
      float tmpvar_9;
      tmpvar_9 = clamp ((1.0 - (uPointLightingInvRadii[i_1] * 
        sqrt(dot (tmpvar_8, tmpvar_8))
      )), 0.0, 1.0);
      vec3 tmpvar_10;
      tmpvar_10 = normalize(tmpvar_8);
      lightWeighting_2 = (lightWeighting_2 + ((uPointLightingColors[i_1] * 
        max (dot (normal_7, tmpvar_10), 0.0)
      ) * tmpvar_9));
      vec3 I_11;
      I_11 = -(tmpvar_10);
      specularHighlight_3 = (specularHighlight_3 + ((
        (specularValue_4 * uPointLightingColors[i_1])
       * 
        pow (max (dot ((I_11 - 
          (2.0 * (dot (normal_7, I_11) * normal_7))
        ), eyeDirection_6), 0.0), specularIntensity_5)
      ) * tmpvar_9));
    };
    if ((i_1 < uDirectionalLightCount)) {
      lightWeighting_2 = (lightWeighting_2 + (uDirectionalLightingColors[i_1] * max (
        dot (normal_7, uDirectionalLightingDirections[i_1])
      , 0.0)));
    };
    if ((i_1 < uSpotLightCount)) {
      vec3 tmpvar_12;
      tmpvar_12 = (uSpotLightingPositions[i_1] - vPosition.xyz);
      float tmpvar_13;
      tmpvar_13 = clamp ((1.0 - (uSpotLightingInvRadii[i_1] * 
        sqrt(dot (tmpvar_12, tmpvar_12))
      )), 0.0, 1.0);
      vec3 tmpvar_14;
      tmpvar_14 = normalize(tmpvar_12);
      float tmpvar_15;
      tmpvar_15 = max (dot (normal_7, tmpvar_14), 0.0);
      float tmpvar_16;
      tmpvar_16 = dot (-(tmpvar_14), normalize(uSpotLightingDirections[i_1]));
      if ((tmpvar_15 > 0.0)) {
        vec3 spotContribution_17;
        spotContribution_17 = vec3(0.0, 0.0, 0.0);
        float tmpvar_18;
        tmpvar_18 = cos(uSpotLightingCutoffs[i_1]);
        float tmpvar_19;
        tmpvar_19 = cos(uSpotLightingOuterCutoffs[i_1]);
        if ((tmpvar_16 > tmpvar_18)) {
          spotContribution_17 = ((uSpotLightingColors[i_1] * tmpvar_15) * tmpvar_13);
        } else {
          if ((tmpvar_16 > tmpvar_19)) {
            spotContribution_17 = (((uSpotLightingColors[i_1] * tmpvar_15) * (
              (tmpvar_16 - tmpvar_19)
             / 
              (tmpvar_18 - tmpvar_19)
            )) * tmpvar_13);
          };
        };
        lightWeighting_2 = (lightWeighting_2 + spotContribution_17);
      };
    };
  };
  //lightWeighting_2 = (lightWeighting_2 + uAmbientValue);
  vec4 tmpvar_20;
  tmpvar_20.xyz = frag.xyz;
  tmpvar_20.w = frag.w;
  gl_FragColor.w = tmpvar_20.w;
  gl_FragColor.xyz = (tmpvar_20.xyz + specularHighlight_3);
  float tmpvar_21;
  tmpvar_21 = (gl_FragCoord.z / gl_FragCoord.w);
  gl_FragColor = mix (uFogColor, gl_FragColor, clamp (exp2(
    ((((
      -(uFogDensity)
     * uFogDensity) * tmpvar_21) * tmpvar_21) * 1.442695)
  ), 0.0, 1.0));
  
  float spread = uWorldHeightCutoffSpread;
  float top = uWorldTopCutoff;
  float bottom = uWorldBottomCutoff;
  
  spread += sin((vPosition.x) * 2.0) * 2.0;
  
	if( vPosition.y > top - spread &&
		vPosition.y < top)
	{
		gl_FragColor.a *= pow(abs(vPosition.y - top),2.0)/spread;
	}
	else if(vPosition.y < bottom + spread &&
		vPosition.y > bottom)
	{
		gl_FragColor.a *= pow(abs(vPosition.y - bottom),2.0)/spread;
	}
}