#version 150

attribute vec3 aVertexPosition;
attribute vec3 aVertexNormal;
attribute vec2 aTextureCoord;

uniform mat4 uModelMatrix;
uniform mat4 uViewMatrix;
uniform mat4 uProjectionMatrix;
uniform mat3 uNormalMatrix;

varying vec2 vTextureCoord;
varying vec3 vTransformedNormal;
varying vec4 vPosition;

uniform float uPerpendicularOffsetBaseLevel;
uniform float uPerpendicularOffsetTopLevel;
uniform float uPerpendicularOffsetMagnitude;
uniform vec3 uPerpendicularOffsetDirection;

uniform float uWaveMagnitude;
uniform vec3 uWaveDirection;
uniform vec3 uWaveFalloffAxis;
uniform float uWaveFalloffBase;
uniform float uWaveFalloffTop;
uniform float uSine;
uniform float uOffset;

void main(void)
{
    vec3 dir = aVertexPosition * uPerpendicularOffsetDirection;
    float len = sqrt(dir.x*dir.x + dir.y*dir.y + dir.z*dir.z);
    float perpOffset = 0.0;

    if(uPerpendicularOffsetTopLevel != uPerpendicularOffsetBaseLevel)
    {
        perpOffset = uPerpendicularOffsetMagnitude * ((len-uPerpendicularOffsetBaseLevel)/(uPerpendicularOffsetTopLevel-uPerpendicularOffsetBaseLevel));
    }

    float mag = (((sin(uSine + perpOffset) + 1.0) * 0.5) + uOffset) * uWaveMagnitude;

    vec3 falloffAxis = aVertexPosition * uWaveFalloffAxis;
    float falloffLength = sqrt(falloffAxis.x*falloffAxis.x + falloffAxis.y*falloffAxis.y + falloffAxis.z*falloffAxis.z);

    float falloff = min(1.0, max(0.0, ((falloffLength-uWaveFalloffBase)/(uWaveFalloffTop-uWaveFalloffBase))));
    float amplitude = min(1.0, max(0.0, falloff*falloff));

    vPosition = vec4(aVertexPosition, 1.0);
    vPosition.xyz += ((amplitude * mag * 0.3) * uWaveDirection);
    vPosition = uModelMatrix * vPosition;

    gl_Position = uProjectionMatrix * uViewMatrix * vPosition;
    vTextureCoord = aTextureCoord;
    vTransformedNormal = uNormalMatrix * aVertexNormal;
}