#version 150

attribute vec3 aVertexPosition;
attribute vec3 aVertexNormal;
attribute vec2 aTextureCoord;

uniform mat4 uModelMatrix;
uniform mat4 uViewMatrix;
uniform mat4 uProjectionMatrix;
uniform mat3 uNormalMatrix;

varying vec2 vTextureCoord;
varying vec4 vPosition;
varying vec3 vLightWeighting;

uniform float uPerpendicularOffsetBaseLevel;
uniform float uPerpendicularOffsetTopLevel;
uniform float uPerpendicularOffsetMagnitude;
uniform vec3 uPerpendicularOffsetDirection;

uniform float uWaveMagnitude;
uniform vec3 uWaveDirection;
uniform vec3 uWaveFalloffAxis;
uniform float uWaveFalloffBase;
uniform float uWaveFalloffTop;
uniform float uSine;
uniform float uOffset;
uniform float uWavelength;

uniform vec3 uPointLightingPositions[15];
uniform vec3 uPointLightingColors[15];
uniform float uPointLightingInvRadii[15];
uniform int uPointLightCount;
uniform vec3 uDirectionalLightingDirections[15];
uniform vec3 uDirectionalLightingColors[15];
uniform int uDirectionalLightCount;
uniform vec3 uSpotLightingPositions[15];
uniform vec3 uSpotLightingDirections[15];
uniform vec3 uSpotLightingColors[15];
uniform float uSpotLightingInvRadii[15];
uniform float uSpotLightingCutoffs[15];
uniform float uSpotLightingOuterCutoffs[15];
uniform int uSpotLightCount;
uniform vec3 uAmbientValue;

void main(void)
{
    vec3 dir = aVertexPosition * uPerpendicularOffsetDirection;
    float len = sqrt(dir.x*dir.x + dir.y*dir.y + dir.z*dir.z);
    float perpOffset = 0.0;

    if(uPerpendicularOffsetTopLevel != uPerpendicularOffsetBaseLevel)
    {
        perpOffset = uPerpendicularOffsetMagnitude * ((len-uPerpendicularOffsetBaseLevel)/(uPerpendicularOffsetTopLevel-uPerpendicularOffsetBaseLevel));
    }

    float mag = (((sin(uSine + perpOffset) + 1.0) * 0.5) + uOffset) * uWaveMagnitude;

    vec3 falloffAxis = aVertexPosition * uWaveFalloffAxis;
    float falloffLength = sqrt(falloffAxis.x*falloffAxis.x + falloffAxis.y*falloffAxis.y + falloffAxis.z*falloffAxis.z);

    float falloff = min(1.0, max(0.0, ((falloffLength-uWaveFalloffBase)/(uWaveFalloffTop-uWaveFalloffBase))));
    float amplitude = min(1.0, max(0.0, falloff*falloff));

    vPosition = vec4(aVertexPosition, 1.0);
    vPosition.xyz += ((amplitude * mag * 0.3) * uWaveDirection);
	vPosition = uProjectionMatrix * uViewMatrix * uModelMatrix * vPosition;
	
    gl_Position = vPosition;
    vTextureCoord = aTextureCoord;
    
	
	vec3 normal = normalize(uNormalMatrix * aVertexNormal);

			vLightWeighting = vec3(0,0,0);
			for(int i = 0; i < 15; i++)
			{
			float att = 0.0;

			if(i < uPointLightCount)
			{
			vec3 pointLightDirection = (uPointLightingPositions[i] - vPosition.xyz);
			att = clamp(1.0 - uPointLightingInvRadii[i] * sqrt(dot(pointLightDirection, pointLightDirection)), 0.0, 1.0);
			pointLightDirection = normalize(pointLightDirection);
			float pointLightWeighting = max(dot(normal, pointLightDirection), 0.0);
			vLightWeighting += uPointLightingColors[i] * pointLightWeighting * att;

			}

			if(i < uDirectionalLightCount)
			{
			float directionalLightWeighting = max(dot(normal, uDirectionalLightingDirections[i]), 0.0);
			vLightWeighting += (uDirectionalLightingColors[i] * directionalLightWeighting);
			}

			if(i < uSpotLightCount)
			{
			vec3 directionToSpot = (uSpotLightingPositions[i] - vPosition.xyz);
			att = clamp(1.0 - uSpotLightingInvRadii[i] * sqrt(dot(directionToSpot, directionToSpot)), 0.0, 1.0);
			directionToSpot = normalize(directionToSpot);
			vec3 spotDirection = normalize(uSpotLightingDirections[i]);
			float spotLightWeighting = max(dot(normal, directionToSpot), 0.0);
			float spotAngle = dot(-directionToSpot, spotDirection);

			if(spotLightWeighting > 0.0)
			{
			vec3 spotContribution = vec3(0,0,0);
			float innerCutoffCosine = cos(uSpotLightingCutoffs[i]);
			float outerCutoffCosine = cos(uSpotLightingOuterCutoffs[i]);

			if( spotAngle > innerCutoffCosine )
			{
			spotContribution = uSpotLightingColors[i] * spotLightWeighting * att;
			}
			else if( spotAngle > outerCutoffCosine )
			{
			float falloff = (spotAngle - outerCutoffCosine) / (innerCutoffCosine - outerCutoffCosine);
			spotContribution = uSpotLightingColors[i] * spotLightWeighting * falloff * att;
			}

			vLightWeighting += spotContribution;
			}
			}
			}

			vLightWeighting += uAmbientValue;
}