#version 150

varying vec2 vTextureCoord;
varying vec4 vPosition;
varying vec3 vTransformedNormal;

uniform vec4 uDiffuseValue;
uniform vec3 uSpecularValue;
uniform float uSpecularIntensity;
uniform sampler2D uDiffuseSampler;
uniform float uAlphaMultiplier;

uniform float uMagnitude;
uniform vec2 uScreenSize;

uniform vec3 uAmbientValue;
uniform vec3 uPointLightingPositions[10];
uniform vec3 uPointLightingColors[10];
uniform float uPointLightingInvRadii[10];
uniform int uPointLightCount;
uniform vec3 uDirectionalLightingDirections[10];
uniform vec3 uDirectionalLightingColors[10];
uniform int uDirectionalLightCount;
uniform vec3 uSpotLightingPositions[10];
uniform vec3 uSpotLightingDirections[10];
uniform vec3 uSpotLightingColors[10];
uniform float uSpotLightingInvRadii[10];
uniform float uSpotLightingCutoffs[10];
uniform float uSpotLightingOuterCutoffs[10];
uniform int uSpotLightCount;
uniform vec4 uFogColor;
uniform float uFogDensity;
uniform vec3 uCameraPosition;

uniform sampler2D uDepthSampler;
uniform float uNearPlane;
uniform float uFarPlane;
uniform float uDepthFalloff;

void main(void)
{
    vec2 displace = vTransformedNormal.xy;
    displace *= uMagnitude;
    displace /= uScreenSize;
	
	vec4 background = texture2D(uDiffuseSampler, (gl_FragCoord.xy / uScreenSize) + displace);
	background.rgb *= uDiffuseValue.rgb;
	
	background.rgb = mix(background.rgb, uDiffuseValue.rgb, uDiffuseValue.a);
	
    vec3 lightWeighting_2;
  vec3 specularHighlight_3;
  vec3 specularValue_4;
  float specularIntensity_5;
  vec3 eyeDirection_6;
  vec3 normal_7;
  normal_7 = normalize(vTransformedNormal);
  eyeDirection_6 = normalize((uCameraPosition - vPosition.xyz));
  specularIntensity_5 = uSpecularIntensity;
  specularValue_4 = uSpecularValue;
  specularHighlight_3 = vec3(0.0, 0.0, 0.0);
  lightWeighting_2 = vec3(0.0, 0.0, 0.0);
  for (int i_1 = 0; i_1 < 10; i_1++) {
    if ((i_1 < uPointLightCount)) {
      vec3 tmpvar_8;
      tmpvar_8 = (uPointLightingPositions[i_1] - vPosition.xyz);
      float tmpvar_9;
      tmpvar_9 = clamp ((1.0 - (uPointLightingInvRadii[i_1] * 
        sqrt(dot (tmpvar_8, tmpvar_8))
      )), 0.0, 1.0);
      vec3 tmpvar_10;
      tmpvar_10 = normalize(tmpvar_8);
      lightWeighting_2 = (lightWeighting_2 + ((uPointLightingColors[i_1] * 
        max (dot (normal_7, tmpvar_10), 0.0)
      ) * tmpvar_9));
      vec3 I_11;
      I_11 = -(tmpvar_10);
      specularHighlight_3 = (specularHighlight_3 + ((
        (specularValue_4 * uPointLightingColors[i_1])
       * 
        pow (max (dot ((I_11 - 
          (2.0 * (dot (normal_7, I_11) * normal_7))
        ), eyeDirection_6), 0.0), specularIntensity_5)
      ) * tmpvar_9));
    };
    if ((i_1 < uDirectionalLightCount)) {
      lightWeighting_2 = (lightWeighting_2 + (uDirectionalLightingColors[i_1] * max (
        dot (normal_7, uDirectionalLightingDirections[i_1])
      , 0.0)));
    };
    if ((i_1 < uSpotLightCount)) {
      vec3 tmpvar_12;
      tmpvar_12 = (uSpotLightingPositions[i_1] - vPosition.xyz);
      float tmpvar_13;
      tmpvar_13 = clamp ((1.0 - (uSpotLightingInvRadii[i_1] * 
        sqrt(dot (tmpvar_12, tmpvar_12))
      )), 0.0, 1.0);
      vec3 tmpvar_14;
      tmpvar_14 = normalize(tmpvar_12);
      float tmpvar_15;
      tmpvar_15 = max (dot (normal_7, tmpvar_14), 0.0);
      float tmpvar_16;
      tmpvar_16 = dot (-(tmpvar_14), normalize(uSpotLightingDirections[i_1]));
      if ((tmpvar_15 > 0.0)) {
        vec3 spotContribution_17;
        spotContribution_17 = vec3(0.0, 0.0, 0.0);
        float tmpvar_18;
        tmpvar_18 = cos(uSpotLightingCutoffs[i_1]);
        float tmpvar_19;
        tmpvar_19 = cos(uSpotLightingOuterCutoffs[i_1]);
        if ((tmpvar_16 > tmpvar_18)) {
          spotContribution_17 = ((uSpotLightingColors[i_1] * tmpvar_15) * tmpvar_13);
        } else {
          if ((tmpvar_16 > tmpvar_19)) {
            spotContribution_17 = (((uSpotLightingColors[i_1] * tmpvar_15) * (
              (tmpvar_16 - tmpvar_19)
             / 
              (tmpvar_18 - tmpvar_19)
            )) * tmpvar_13);
          };
        };
        lightWeighting_2 = (lightWeighting_2 + spotContribution_17);
      };
    };
  };
  lightWeighting_2 = (lightWeighting_2 + uAmbientValue);
  vec4 tmpvar_20;
  tmpvar_20.xyz = (background.rgb * lightWeighting_2);
  tmpvar_20.w = uAlphaMultiplier;
  gl_FragColor.w = tmpvar_20.w;
  gl_FragColor.xyz = (tmpvar_20.xyz + specularHighlight_3);
  float tmpvar_21;
  tmpvar_21 = (gl_FragCoord.z / gl_FragCoord.w);
  gl_FragColor = mix (uFogColor, gl_FragColor, clamp (exp2(
    ((((
      -(uFogDensity)
     * uFogDensity) * tmpvar_21) * tmpvar_21) * 1.442695)
  ), 0.0, 1.0));
  
  
    vec4 depth = texture2D(uDepthSampler, (gl_FragCoord.xy / uScreenSize));
    float z_n = 2.0 * depth.r - 1.0;
    float linearWorldDepth = 2.0 * uNearPlane * uFarPlane / (uFarPlane + uNearPlane - z_n * (uFarPlane - uNearPlane));
	
    float ndcDepth = 2.0 * uNearPlane * uFarPlane / (uFarPlane + uNearPlane - gl_FragCoord.z * (uFarPlane - uNearPlane));
	float fragDepth = ndcDepth;
	
	float difference = (linearWorldDepth - fragDepth);
	
	if(difference < 0.0)
	{
		gl_FragColor.a = 0.0;
	}
	else if(difference < uDepthFalloff)
	{
		gl_FragColor.rgb /= pow(difference / uDepthFalloff, 2.0);
		gl_FragColor.a *= difference / uDepthFalloff;
	}
}